#############################################################
#
# ARRANGER
#
#############################################################
class CADataFrame
  class Arranger

    def initialize (dataframe)
      @dataframe = dataframe
    end

    def arrange (&block)
      case block.arity
      when 1
        instance_exec(@dataframe, &block)
      else
        instance_exec(&block)
      end
      return @dataframe
    end
    private

    def index
      return @dataframe.index
    end

    def column_names
      return @dataframe.column_names
    end

    def row_number
      return @dataframe.row_number
    end

    def row_index
      return @dataframe.row_index
    end

    def method (hash)
      @dataframe.method(hash)
    end

    def timeseries (name, format)
      time_column = @dataframe.columns[name.to_s]
      column = CATimeIndex.from_time_array(time_column, "sec", format: format)
      @dataframe.columns[name.to_s] = column
    end

    def timeindex (name, unit: "second", since: nil, offset: nil, format: nil)
      time_column = @dataframe.columns[name.to_s]
      column = CATimeIndex.from_time_array(time_column, unit, since: since, format: format, offset: offset)
      @dataframe.columns[name.to_s] = column
    end

    def type (type, name, mask = :novalue)
      @dataframe.columns[name.to_s] = @dataframe.columns[name.to_s].to_type(type)
      if mask != :novalue
        @dataframe.columns[name.to_s].maskout!(options[:maskout])
      end
    end

    def eliminate (*names)
      if names.empty?
        return self
      end
      names = names.map(&:to_s)
      @dataframe.column_names.clone.each do |name|
        if names.include?(name)
          @dataframe.columns.delete(name)
          @dataframe.column_names.delete(name)
        end
      end
    end
    
    alias drop eliminate

    def template (*args, &block)
      return @dataframe.columns.first[1].template(*args, &block)
    end

    def double (*names)
      names.flatten.map(&:to_s).each do |name|
        if @dataframe.column_names.include?(name)
          type(:double, name)
        else
          raise "Unknown column name '#{name}'"
        end
      end
    end

    def int (*names)
      names.flatten.map(&:to_s).each do |name|
        if @dataframe.column_names.include?(name)
          type(:int, name)
        else
          raise "Unknown column name '#{name}'"
        end
      end
    end

    def maskout (value, *names)
      warn "maskout is obsolete. use mask(name, *values)"
      names.flatten.map(&:to_s).each do |name|
        @dataframe.columns[name].maskout!(value)
      end
    end

    def mask (name, *values)
      column = @dataframe.columns[name.to_s]
      values.each do |args|
        if args.nil?
          column.maskout!(nil)
        else
          column.maskout!(*args)
        end
      end
    end

    def unmask (value, *names)
      names.flatten.map(&:to_s).each do |name|
        @dataframe.columns[name].unmask(value)
      end
    end

    def col (name)
      return @dataframe.col(name)
    end

    def append (name, new_column)
      if new_column
        # do nothing
      else
        new_column = @dataframe.columns.first[1].template(:object)
      end
      unless new_column.is_a?(CArray)
        new_column = new_column.to_ca
      end
      @dataframe.columns[name.to_s] = new_column
      @dataframe.column_names.push(name.to_s)
    end

    def prepend (name, new_column)
      if new_column
        # do nothing
      else
        new_column = @dataframe.columns.first[1].template(:object)
      end
      unless new_column.is_a?(CArray)
        new_column = new_column.to_ca
      end
      @dataframe.columns[name.to_s] = new_column
      @dataframe.column_names.unshift(name.to_s)
    end
    alias lead prepend

    def rename (name1, name2)
      if idx = @dataframe.column_names.index(name1.to_s)
        @dataframe.column_names[idx] = name2.to_s
        column = @dataframe.columns[name1.to_s]
        @dataframe.columns.delete(name1.to_s)
        @dataframe.columns[name2.to_s] = column
      else
        raise "unknown column name #{name1}"
      end
    end

    def downcase
      @dataframe.downcase
    end

    def classify (name, scale, opt = {})
      return @dataframe.classify(name, scale, opt)
    end

    def map (mapper, name_or_column)
      case name_or_column
      when String, Symbol
        name = name_or_column
        column = @dataframe.columns[name.to_s]
      when CArray
        column = name_or_column
      when Array
        column = name_or_column.to_ca
      else
        raise "invalid argument"
      end
      case mapper
      when Hash
        return column.convert(:object) {|v| hash[v] }
      when CArray
        return mapper.project(column)
      when Array
        return mapper.to_ca.project(column)
      end
    end

    def method_missing (name, *args)
      if args.size == 0
        if @dataframe.column_names.include?(name.to_s)
          return @dataframe.columns[name.to_s]
        elsif @dataframe.__methods__.include?(name.to_s)
          return @dataframe.columns[@dataframe.__methods__[name.to_s]]
        end
      end
      super
    end
  end
end