#############################################################
#
# CADFArray
#
#############################################################
class CADFArray < CAObject # :nodoc:

  def initialize (column_names, column_data, index: nil)
    @column_names = column_names
    @column_data  = column_data
    if index
      @index = index
    else
      @index = CArray.int(column_data.first[1].size).seq
    end
    dim = [@column_data[@column_names.first].size, @column_names.size]
    extend CArray::TableMethods
    super(:object, dim, :read_only=>true)
    __create_mask__
  end
  
  attr_reader :column_names, :index

  def fetch_index (idx)
    r, c = *idx
    name = @column_names[c]
    return @column_data[name].value[r]
  end

  def store_index (idx, value)
    r, c = *idx
    name = @column_names[c]
    return @column_data[name][r] = value
  end

  def copy_data (data)
    @column_names.each_with_index do |name, i|
      data[nil,i] = @column_data[name].value
    end
  end

  def sync_data (data)
    @column_names.each_with_index do |name, i|
      @column_data[name].value[] = data[nil,i]
    end
  end

  def fill_data (value)
    @column_names.each do |name|
      @column_data[name] = value
    end
  end

  def create_mask
    @column_names.each do |name|
       @column_data[name].instance_eval{ __create_mask__ }
    end
  end

  def mask_fetch_index (idx)
    r, c = *idx
    name = @column_names[c]
    if @column_data[name].has_mask?
      return @column_data[name].mask[r]
    else
      return 0
    end
  end

  def mask_store_index (idx, value)
    r, c = *idx
    name = @column_names[c]
    if @column_data[name].has_mask?
      return @column_data[name].mask[r] = value
    else
      @column_data[name].mask[r] = value
    end
  end

  def mask_copy_data (data)
    @column_names.each_with_index do |name, i|
      if @column_data[name].has_mask?
        data[nil,i] = @column_data[name].mask
      end
    end
  end

  def mask_sync_data (data)
    @column_names.each_with_index do |name, i|
      @column_data[name].mask[] = data[nil,i]
    end
  end

  def mask_fill_data (value)
    @column_names.each do |name|
      @column_data[name].mask[] = value
    end
  end

  def to_ca
    obj = super
    obj.extend CArray::TableMethods
    obj.column_names = @column_names
    return obj
  end
end