
module CArray::TableMethods

  def to_dataframe (index: nil, &block)
    if self.size == 0
      return nil
    end
    df = CADataFrame.new(self, index: index, &block)
    if @header or @note
      df.instance_variable_set(:@header, @header)
      df.instance_variable_set(:@note, @note)
      class << df
        attr_reader :note

        def header (name=nil)
          if name
            return @header[name.to_s]
          else
            return @column_names
          end
        end
      end
    end
    return df
  end
  alias to_df to_dataframe
end

class CADataFrame

  def to_a (with_index: true)
    if @row_index and with_index
      namelist = [""] + @column_names
      tbl = CADFArray.new(namelist, @column_data.clone.update("" => index))
    else
      tbl = ca.to_ca
    end
    return tbl.to_a
  end

  def to_csv (io = "", rs: $/, sep: ",", fill: "", with_index: true, time_format: nil, &block)
    if @row_index and with_index
      namelist = ["index"] + @column_names
      columns = @column_data.clone.update("index" => index)
    else
      namelist = @column_names
      columns = @column_data.clone
    end
    columns.each do |k, v|
      if v.is_a?(CATimeIndex)
        if time_format
          columns[k] = v.time.time_format(time_format)
        else
          columns[k] = v.time.convert(:object){|t| t.to_s}
        end
      end
    end
    tbl = CADFArray.new(namelist, columns)
    return tbl.to_csv(io, **{rs: rs, sep: sep, fill: fill}, &block)
  end

  def to_daru
    require "daru"
    columns = {}
    each_column_name do |name|
      columns[name] = column(name).object.unmask(nil).to_a
    end
    if @row_index
      return Daru::DataFrame.new(columns, index: @row_index.to_a, order: @column_names)
    else
      return Daru::DataFrame.new(columns, order: @column_names)
    end
  end

  def to_xlsx (filename, sheet_name: 'Sheet1', with_row_index: false, &block)
    require "caxlsx"
    xl = Axlsx::Package.new
    xl.use_shared_strings = true
    sheet = xl.workbook.add_worksheet(name: sheet_name)
    df = self.to_df.objectify.unmask("=NA()")
    if with_row_index
      sheet.add_row([""] + column_names)
      df.each_row_with_row_index(with: Array) do |list, i|
        sheet.add_row([i] + list)
      end
    else
      sheet.add_row(column_names)
      df.each_row(with: Array) do |list|
        sheet.add_row(list)
      end
    end
    if block_given?
      yield sheet
    end
    xl.serialize(filename)
  end
end