#############################################################
#
# GROUPING
#
#############################################################

class CADataFrame

  def group_by (*names)
    if names.size == 1
      return CADataFrameGroup.new(self, names[0])
    else
      return CADataFrameGroupMulti.new(self, *names)
    end
  end
end

class CADataFrameGroup

  def initialize (dataframe, name)
    @dataframe = dataframe
    case name
    when Hash
      name, list = name.first
      @column = @dataframe.col(name)
      @keys = list.to_ca
    else
      @column = @dataframe.col(name)
      @keys = @column.uniq.sort
    end
    if @column.is_a?(CATimeIndex)
      @keys = CATimeIndex.from_index_array(@keys, @column.timestep)
    end
    @name = name.to_s
    @addrs = {}
    @keys.each do |k|
      @addrs[k] = @column.eq(k).where
    end
  end

  def table (&block)
    hashpool = []
    @keys.each do |k|
      hashpool << @dataframe[@addrs[k]].execute(&block)
    end
    columns = { @name => @keys }
    hashpool.each_with_index do |hash, i|
      hash.each do |key, value|
        columns[key] ||= []
        columns[key][i] = value
      end
    end
    return CADataFrame.new(columns)
  end

	def calculate (label = nil, columns: nil, &block)
    new_columns = { @name => @keys }
		@dataframe.each_column do |name, clmn|
      if name == @name or ( columns && ( not columns.include?(name) ) )
        next
      end
      new_columns[name] = CArray.object(@keys.size) { UNDEF }
      @keys.each_with_index do |k, i|
        begin
          if block
            new_columns[name][i] = yield(name, clmn[@addrs[k]])
          else
            new_columns[name][i] = clmn[@addrs[k]].send(label.intern)
          end
        rescue
        end
      end
    end
    return CADataFrame.new(new_columns)
	end

  def [] (group_value)
    if @column.is_a?(CATimeIndex) and group_value.is_a?(String)
      group_value = @column.timestep.index_at(group_value)
    end
    if map = @addrs[group_value]
      return @dataframe[map]
    else
      return @dataframe.vacant_copy
    end
  end

  def each
    @addrs.each do |key, map|
      yield @dataframe[map]
    end
  end

  def each_with_index
    if @column.is_a?(CATimeIndex)
      ts = @column.timestep
      @addrs.each do |key, map|
        yield @dataframe[map], ts.time_at(key)
      end
    else
      @addrs.each do |key, map|
        yield @dataframe[map], key
      end
    end
  end
  include Enumerable
end
class CADataFrameGroupMulti

  def initialize (dataframe, *names)
    @rank = names.size
    @dataframe = dataframe
    @names = []
    @column = []
    @keys = []
    names.each_with_index do |name, i|
      case name
      when Hash
        name, list = name.first
        @column[i] = @dataframe.col(name)
        @keys[i] = list.to_ca
      else
        @column[i] = @dataframe.col(name)
        @keys[i] = @column[i].to_ca.uniq.sort
      end
      @names[i] = name
    end
    @addrs = {}
    each_with_keys do |list|
      flag = @column[0].eq(list[0])
      (1...@rank).each do |i|
        flag &= @column[i].eq(list[i])
      end
      @addrs[list] = flag.where
    end
  end

  def each_with_keys (&block)
    @keys[0].to_a.product(*@keys[1..-1].map(&:to_a)).each(&block)
  end

  def table (&block)
    hashpool = []
    each_with_keys do |list|
      hashpool << @dataframe[@addrs[list]].execute(&block)
    end
    columns = {}
    @names.each do |name|
      columns[name] = []
    end
    each_with_keys.with_index do |list,j|
      @names.each_with_index do |name,i|
        columns[name][j] = list[i]
      end
    end
    hashpool.each_with_index do |hash, i|
      hash.each do |key, value|
        columns[key] ||= []
        columns[key][i] = value
      end
    end
    return CADataFrame.new(columns)
  end

	def calculate (label, &block)
    new_columns = {@name=>@keys}
		@dataframe.each_column do |name, clmn|
      if name == @name
        next
      end
      new_columns[name] = CArray.object(@keys.size) { UNDEF }
      @keys.each_with_index do |k, i|
        begin
          if block
            new_columns[name][i] = yield(name, clmn[@addrs[k]])
          else
            new_columns[name][i] = clmn[@addrs[k]].send(label.intern)
          end
        rescue
        end
      end
    end
    return CADataFrame.new(new_columns)
	end

  def [] (group_value)
    if map = @addrs[group_value]
      return @dataframe[map]
    else
      return @dataframe.vacant_copy
    end
  end

  def each
    each_with_keys do |key|
      yield key, @dataframe[@addrs[key]]
    end
  end
end