######################################
#
#  IO methods
#
######################################
require "spreadsheet"
require "carray-io-sqlite3"

class CArray

  def save_excel (filename, &block)
    if self.rank >= 3
      raise "too large rank (>2) to write excel file"
    end
    book = Spreadsheet::Workbook.new
    worksheet = book.create_worksheet
    self.dim0.times do |i|
      worksheet.row(i).push *self[i,nil]
    end
    if block
      block.call(worksheet)
    end
    book.write(filename)
  end

  def self.load_excel (filename, sheet=0)
    book = Spreadsheet.open(filename)
    sheet = book.worksheet(sheet)
    return sheet.map(&:to_a).to_ca
  end
end
class CADataFrame

  def self.load_sqlite3 (*args)
    df = CArray.load_sqlite3(*args).to_dataframe
    if df
      return df.arrange{ 
        column_names.each do |name|
          mask name, nil
        end
      }
    else
      return nil
    end
  end

  def self.read_csv (file, sep: ",", rs: $/, quote_char: '"', encoding: nil, index: nil, &block)
    df = CArray.read_csv(file, sep: sep, rs: rs, quote_char: quote_char, encoding: encoding, &block).to_dataframe(index: index)
    if df
      return df.arrange{ 
        column_names.each do |name|
          mask name, nil
        end
      }
    else
      return nil
    end
  end

  def self.load_csv (file, sep: ",", rs: $/, encoding: nil, quote_char: '"', index: nil, &block)
    warn "CADataFrame.load_csv will be obsolete, use CADataFrame.read_csv"
    self.read_csv(file, sep: sep, rs: rs, quote_char: quote_char, encoding: encoding, &block)
  end

  def self.parse_csv (file, sep: ",", rs: $/, quote_char: '"', index: nil, &block)
    df = CArray.parse_csv(file, sep: sep, rs: rs, quote_char: quote_char, &block).to_dataframe(index: index)
    if df
      return df.arrange{ 
        column_names.each do |name|
          mask name, nil
        end
      }
    else
      return nil
    end
  end

  def self.from_csv (file, sep: ",", rs: $/, quote_char: '"', index: nil, &block)
    warn "CADataFrame.from_csv will be obsolete, use CADataFrame.parse_csv"
    self.parse_csv(file, sep: sep, rs: rs, index: index, &block)
  end


  def to_sqlite3 (**args)
    self.to_ca.to_sqlite3(**args)
  end

  def to_sql (tablename)
    if @column_names.any?{ |s| s =~ /[\. \-]/ }
      columns = {}
      each_column_name do |name|
        name2 = name.gsub(/[\. \-]/, '_')
        columns[name2] = column(name)
      end
      df = CADataFrame.new(columns)
      return df.to_sqlite3(database: ":memory:", table: tablename)
    else
      return to_sqlite3(database: ":memory:", table: tablename)
    end
  end
end
module SQLite3
  class Database

    def to_df (expr)
      return CADataFrame.load_sqlite3 self, expr
    end
  end
end