
class CADataFrame

  def select_columns (selector = nil)
    case selector
    when nil                       ### all
      return @column_names
    when Integer
      name = @column_names[selector]
      raise "invalid column index" unless selector
      return [name]
    when String, Symbol            ### "AAA"
      if @column_names.include?(selector.to_s)
        return [selector.to_s]
      else
        raise "invalid column specified #{selector}"
      end
    when Array                     ### ["AAA", "BBB"]
      if selector.size == 1 && selector.first.is_a?(Hash)
        return select_columns(selector.first)
      else
        selector.each do |name|
          unless @column_names.include?(name.to_s)
            raise "invalid column specified #{name}"
          end
        end
        return selector.map(&:to_s)
      end
    when Range                     ### "AAA".."BBB", 0..1
      case selector.begin
      when nil
        idx1 = 0
      when Integer
        idx1 = selector.begin
      when String, Symbol
        idx1 = @column_names.search(selector.begin.to_s)
        raise "can't find column #{selector.begin}" unless idx1
      else
        raise "invalid column specified #{selector.begin}"
      end
      if selector.exclude_end?
        case selector.end
        when nil
          idx2 = -2
        when Integer
          idx2 = selector.end - 1
        when String
          idx2 = @column_names.search(selector.end.to_s) 
          raise "can't find column #{selector.end}" unless idx2
          idx2 = idx2 - 1
        else
          raise "invalid column specified #{selector.end}"
        end        
      else
        case selector.end
        when nil
          idx2 = -1
        when Integer
          idx2 = selector.end
        when String
          idx2 = @column_names.search(selector.end.to_s)
          raise "can't find column #{selector.end}" unless idx2
        else
          raise "invalid column specified #{selector.end}"
        end
      end
      return @column_names[idx1..idx2]
    else
      raise "invalid column selector #{selector}"              
    end
  end

  def [] (arg, opt = :__dummy__)
    if opt != :__dummy__
      return loc[arg, opt]
    else
      case arg
      when Range
        if arg.begin.is_a?(Integer)
          return iloc[arg] unless @row_index
        end
        return loc[arg]
      when CArray
        if arg.rank == 1
          return loc[arg]
        else
          raise "index should be 1-dim array"
        end
      when String, Symbol
        return column(arg.to_s)
      else
        column_selector = select_columns(arg)
        new_columns = {}
        column_selector.each do |key|
          new_columns[key] = @column_data[key]
        end
        return CADataFrame.new(new_columns, index: @row_index)
      end
    end
  end

  def []= (arg, opt = :__dummy__, value)
    if opt != :__dummy__
      loc[arg, opt] = value
    else
      case arg
      when Range
        if arg.begin.is_a?(Integer)
          iloc[arg] = value unless @row_index
        end
        loc[arg] = value
      when CArray
        loc[arg] = value
      when String, Symbol
        if column(arg.to_s)
          column(arg.to_s)[] = value
        else
          arrange {
            append arg, value
          }
        end
      else
        case value
        when CADataFrame
          column_selector = select_columns(arg)
          values = column_selector.each_index.map { |i| 
            value.column(i).to_ca
          }
          column_selector.each_with_index do |key, i|
            column(key)[] = values[i]
          end
        else
          column_selector = select_columns(arg)
          column_selector.each do |key|
            column(key)[] = value
          end        
        end
      end
    end
  end
  
end