class CADataFrame

  def to_html (threshold = 8, time_format: nil, index: true)
    columns = @column_data.clone
    @column_names.each do |name|
      if columns[name].is_a?(CATimeIndex)
        if time_format
          columns[name] = columns[name].time.time_strftime(time_format)
        else
          columns[name] = columns[name].time.time_format("%F %T%:z")
        end
      end
    end
    if index
      if @row_index
        namelist = ["    "] + @column_names
        if @row_index.is_a?(CATimeIndex)
          if time_format
            row_index = @row_index.time.time_strftime(time_format)
          else
            row_index = @row_index.time.time_format("%F %T%:z")
          end
        else
          row_index = @row_index
        end
        tbl = CADFArray.new(namelist, columns.update("    " => row_index))
      else
        namelist = ["    "] + @column_names
        tbl = CADFArray.new(namelist, columns.update("    " => CArray.int(@row_number).seq))
      end
    else
      namelist = @column_names
      tbl = CADFArray.new(namelist, columns)
    end
    if threshold.is_a?(Integer) and @row_number > threshold
      list = tbl[0..(threshold/2),nil].to_a
      list.push namelist.map { "..." }
      list.push *(tbl[-threshold/2+1..-1,nil].to_a)
      tbl = list.to_ca
    end
    datastr = tbl.convert {|c| __obj_to_string__(c) }.unmask("")
    datamb  = datastr.convert(:boolean, &:"ascii_only?").not.sum(0).ne(0)
    namemb  = namelist.to_ca.convert(:boolean) {|c| c.to_s.ascii_only? }.eq(0)
    mb      = datamb.or(namemb)
    namelen = namelist.map(&:length).to_ca
    datalen = datastr.convert(&:length)
    if mb.max == 0
      if datalen.size == 0
        lengths  = namelen.to_a
      else
        lengths  = datalen.max(0).pmax(namelen).to_a
      end
      table_in = "<table>"
      header = "<thead><tr>" +
               [namelist, lengths].transpose.map{|name, len|
                  "<th>#{name.to_s.ljust(len)}</th>"
               }.join() + "</tr></thead>"
      body_in = "<tbody>"
      ary = [table_in, header, body_in]
			if datalen.size > 0
	      datastr[:i,nil].each_with_index do |blk, i|
	        list = blk.flatten.to_a
	        ary << "<tr>" + [list, lengths].transpose.map {|value, len|
	                        "<td>#{value.ljust(len)}</td>"
                       }.join() + "</tr>"
	      end
			end
      ary << "</tbody>"
      ary << "</table>"
      return "DataFrame: rows#=#{@row_number}: \n" + ary.join("\n")
    else
      namewidth  = namelist.to_ca.convert{|c| __strwidth__(c.to_s) }
      if datalen.size == 0
        maxwidth   = namewidth
      else
        datawidth  = datastr.convert{|c| __strwidth__(c.to_s) }
        maxwidth   = datawidth.max(0).pmax(namewidth)
      end
      len = maxwidth[:*,nil] - datawidth + datalen
      table_in = "<table>"
      header = "<thead><tr>" +
               [namelist, maxwidth.to_a].transpose.map{|name, len|
                 "<th>#{name.to_s.ljust(len-__strwidth__(name.to_s)+name.to_s.length)}</th>"
               }.join() + "</tr></thead>"
      body_in = "<tbody>"
      ary = [table_in, header, body_in]
			if datalen.size > 0
	      datastr[:i,nil].each_with_addr do |blk, i|
	        list = blk.flatten.to_a
	        ary << "<tr>" + list.map.with_index {|value, j|
                       
	                          "<td>#{value.ljust(len[i,j])}</td>"
                          }.join() + "</tr>"
	      end
			end
      ary << "</tbody>"
      ary << "</table>"
      return "DataFrame: row#=#{@row_number}: \n" + ary.join("\n")
    end
  end

end